@@ -1,9 +1,5 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
-#
-# mailchecker.py
-#
-# Copyright 2011 - 2014 Patrick Ulbrich <zulu99@gmx.net>
+# Copyright 2011 - 2020 Patrick Ulbrich <zulu99@gmx.net>
+# Copyright 2020 Andreas Angerer
 # Copyright 2011 Ralf Hersel <ralf.hersel@gmx.net>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -32,14 +28,16 @@ from Mailnag.daemon.mails import MailSyncer
 
 
 class MailChecker:
-	def __init__(self, cfg, memorizer, hookreg, conntest):
+	def __init__(self, cfg, memorizer, hookreg, conntest, dbus_service):
 		self._firstcheck = True # first check after startup
 		self._mailcheck_lock = threading.Lock()
 		self._mailsyncer = MailSyncer(cfg)
+		self._mailbox_seen_flags = bool(cfg.get('core', 'mailbox_seen_flags'))
 		self._memorizer = memorizer
 		self._hookreg = hookreg
 		self._conntest = conntest
-		self._zero_mails_on_last_check = True
+		self._dbus_service = dbus_service
+		self._count_on_last_check = 0
 		
 	
 	def check(self, accounts):
@@ -58,6 +56,7 @@ class MailChecker:
 			all_mails = self._mailsyncer.sync(accounts)
 			unseen_mails = []
 			new_mails = []
+			seen_mails_by_account = {}
 			
 			for mail in all_mails:
 				if self._memorizer.contains(mail.id): # mail was fetched before
@@ -65,11 +64,21 @@ class MailChecker:
 						unseen_mails.append(mail)
 						if self._firstcheck:
 							new_mails.append(mail)
-				
+					elif self._mailbox_seen_flags:
+						# if the mail account supports tagging mails as seen (e.g. IMAP), 
+						# mark the mail as seen on the server as well.
+						if mail.account.supports_mark_as_seen():
+							if not mail.account in seen_mails_by_account:
+								seen_mails_by_account[mail.account] = []
+							seen_mails_by_account[mail.account].append(mail)
 				else: # mail is fetched the first time
 					unseen_mails.append(mail)
 					new_mails.append(mail)
 			
+			# Flag mails to seen on server
+			for acc, mails in seen_mails_by_account.items():
+				acc.mark_as_seen(mails)
+			
 			self._memorizer.sync(all_mails)
 			self._memorizer.save()
 			self._firstcheck = False
@@ -82,14 +91,16 @@ class MailChecker:
 			filtered_new_mails = [m for m in new_mails if m in filtered_unseen_mails]
 			
 			if len(filtered_new_mails) > 0:
+				self._dbus_service.signal_mails_added(filtered_new_mails, filtered_unseen_mails)
+				
 				for f in self._hookreg.get_hook_funcs(HookTypes.MAILS_ADDED):
 					try_call( lambda: f(filtered_new_mails, filtered_unseen_mails) )
-			elif (not self._zero_mails_on_last_check) and (len(filtered_unseen_mails) == 0):
-				# TODO : signal MailsRemoved if not all mails have been removed
-				# (i.e. if mailcount has been decreased)
+			elif len(filtered_unseen_mails) != self._count_on_last_check:
+				self._dbus_service.signal_mails_removed(filtered_unseen_mails)
+				
 				for f in self._hookreg.get_hook_funcs(HookTypes.MAILS_REMOVED):
 					try_call( lambda: f(filtered_unseen_mails) )
 			
-			self._zero_mails_on_last_check = (filtered_unseen_mails == 0)
+			self._count_on_last_check = len(filtered_unseen_mails)
 		
 		return
